@@ -1,7 +1,7 @@
 package Catalyst::View;
 
-use Moose;
-extends qw/Catalyst::Component/;
+use strict;
+use base qw/Catalyst::Component/;
 
 =head1 NAME
 
@@ -52,9 +52,10 @@ Merges two hashes together recursively, giving right-hand precedence.
 
 =cut
 
-=head1 AUTHORS
+=head1 AUTHOR
 
-Catalyst Contributors, see Catalyst.pm
+Sebastian Riedel, C<sri@oook.de>
+Marcus Ramberg, C<mramberg@cpan.org>
 
 =head1 COPYRIGHT
 
@@ -63,7 +64,4 @@ the same terms as Perl itself.
 
 =cut
 
-no Moose;
-__PACKAGE__->meta->make_immutable();
-
 1;
